package com.ruoyi.market.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.idgen.IdGenerate;
import com.ruoyi.market.domain.*;
import com.ruoyi.market.domain.vo.*;
import com.ruoyi.market.mapper.MarketDrawActivityMapper;
import com.ruoyi.market.mapper.MarketDrawTemplateMapper;
import com.ruoyi.market.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class MarketDrawActivityServiceImpl extends ServiceImpl<MarketDrawActivityMapper, MarketDrawActivity>
        implements MarketDrawActivityService {

    @Autowired
    MarketDrawTemplateService marketDrawTemplateService;

    @Autowired
    MarketAwardLibraryService marketAwardLibraryService;

    @Autowired
    MarketDrawActivityProbabilityService marketDrawActivityProbabilityService;

    @Autowired
    MarketUserDrawLogService marketUserDrawLogService;

    @Autowired
    MarketUserDrawService marketUserDrawService;

    /**
     * 获取抽奖活动列表
     *
     * @param page
     * @param drawActivityQueryVo
     * @return
     */
    @Override
    public Page<MarketDrawActivity> getDrawActivityPage(Page page, DrawActivityQueryVo drawActivityQueryVo) {
        LambdaQueryWrapper<MarketDrawActivity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(!Objects.isNull(drawActivityQueryVo.getDrawName()), MarketDrawActivity::getName, drawActivityQueryVo.getDrawName());
        queryWrapper.eq(MarketDrawActivity::getIsDelete, 0);
        return this.page(page, queryWrapper);
    }

    /**
     * 添加抽奖活动
     *
     * @param drawActivitySaveVo
     * @return
     */
    @Transactional
    @Override
    public boolean addDrawActivity(DrawActivitySaveVo drawActivitySaveVo) {
        String drawId = IdGenerate.nextId();
        //验证抽奖模板
        MarketDrawTemplate template = marketDrawTemplateService.getById(drawActivitySaveVo.getDrawActivity().getTemplateId());
        if (Objects.isNull(template)) {
            throw new RuntimeException("抽奖模板不存在");
        }

        //验证奖品
        drawActivitySaveVo.getAwardProbabilityList().forEach(item -> {
            MarketAwardLibrary award = marketAwardLibraryService.getById(item.getId());
            if (!Objects.isNull(award)) {
                throw new RuntimeException("奖品不存在");
            }
        });

        drawActivitySaveVo.getDrawActivity().setId(drawId);

        List<MarketDrawActivityProbability> awardProbabilityList = drawActivitySaveVo.getAwardProbabilityList();
        drawActivitySaveVo.getAwardProbabilityList().forEach(item -> {
            item.setId(IdGenerate.nextId());
            item.setDrawId(drawId);
        });

        List<MarketDrawActivityProbability> newAwardProbabilityList = new ArrayList<>();

        BigDecimal totalProbability = awardProbabilityList.stream()
                .map(MarketDrawActivityProbability::getProbability)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        Integer cha = template.getTemplateDrawNumber() - awardProbabilityList.size();
        //计算谢谢抽奖的其余概率
        BigDecimal radio = new BigDecimal(100).subtract(totalProbability).divide(new BigDecimal(cha), 2, BigDecimal.ROUND_HALF_DOWN);
        for (int i = 1; i <= template.getTemplateDrawNumber(); i++) {
            int finalI = i;
            List<MarketDrawActivityProbability> collect = awardProbabilityList.stream().filter(
                    x -> x.getPos() == finalI).collect(Collectors.toList());
            if (collect.isEmpty()) {
                MarketDrawActivityProbability marketDrawActivityProbability = new MarketDrawActivityProbability();
                marketDrawActivityProbability.setPos(finalI);
                marketDrawActivityProbability.setDrawId(drawId);
                marketDrawActivityProbability.setAwardId("0");
                marketDrawActivityProbability.setProbability(radio);
                marketDrawActivityProbability.setId(IdGenerate.nextId());
                newAwardProbabilityList.add(marketDrawActivityProbability);
            }else{
                collect.get(0).setDrawId(drawId);
                collect.get(0).setId(IdGenerate.nextId());
                newAwardProbabilityList.add(collect.get(0));
            }
        }
        drawActivitySaveVo.setAwardProbabilityList(newAwardProbabilityList);
        marketDrawActivityProbabilityService.saveBatch(drawActivitySaveVo.getAwardProbabilityList());
        this.save(drawActivitySaveVo.getDrawActivity());
        return true;
    }

    @Override
    public MarketDrawActivity getDrawActivityById(String drawActivityId) {
        MarketDrawActivity drawActivity = this.getById(drawActivityId);
        List<MarketDrawActivityProbability> probabilitysByDrawActivityId = marketDrawActivityProbabilityService.getProbabilitysByDrawActivityId(drawActivityId);
        drawActivity.setProbabilitys(probabilitysByDrawActivityId);
        return drawActivity;
    }

    /**
     * 获取前端的抽奖活动信息
     * @return
     */
    @Override
    public List<AwardInfoVo> getForstDrawActivity(String drawActivityId) {
        MarketDrawActivity drawActivity = this.getById(drawActivityId);
        List<AwardInfoVo> awardInfoVos = marketDrawActivityProbabilityService.getDrawAwardListById(drawActivityId);
        return awardInfoVos;
    }

    /**
     * 删除抽奖活动
     *
     * @param id
     * @return
     */
    @Override
    public boolean delDrawActivity(String id) {
        MarketDrawActivity drawActivity = this.getById(id);
        if (Objects.isNull(drawActivity))
            throw new RuntimeException("抽奖活动不存在");
        drawActivity.setIsDelete(1);
        return this.updateById(drawActivity);
    }

    /**
     * 获取抽奖模板下的抽奖活动
     * @param drawTemplateId
     * @return
     */
    @Override
    public List<MarketDrawActivity> getDrawActivityByTemplateId(String drawTemplateId) {
        LambdaQueryWrapper<MarketDrawActivity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarketDrawActivity::getTemplateId, drawTemplateId);
        return this.list(queryWrapper);
    }

    /**
     * 用户抽奖完成
     * @param drawCallBackQueryVo
     * @return
     */
    @Override
    @Transactional
    public boolean userDrawOver(DrawCallBackQueryVo drawCallBackQueryVo) {
        String drawActivityId = drawCallBackQueryVo.getDrawActivityId();
        MarketUserDraw userDrawNumber = marketUserDrawService.getUserDrawNumber(drawCallBackQueryVo.getUserId(), drawActivityId);
        //添加抽奖记录
        MarketUserDrawLog drawLog = new MarketUserDrawLog(
                drawCallBackQueryVo.getUserId(),
                drawActivityId,
                "2", 1,
                userDrawNumber.getAwardNumber() - 1);
        marketUserDrawLogService.save(drawLog);

        //更新用户抽奖次数
        userDrawNumber.setAwardNumber(userDrawNumber.getAwardNumber() - 1);
        marketUserDrawService.updateById(userDrawNumber);

        //发放奖励
        marketAwardLibraryService.grantAward(drawCallBackQueryVo.getDrawActivityId(),drawCallBackQueryVo.getUserId(),drawCallBackQueryVo.getResult());
        return true;
    }

    @Override
    public List<MarketDrawActivity> getDrawActivityList() {
        LambdaQueryWrapper<MarketDrawActivity>  queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarketDrawActivity::getIsDelete,0);
        return this.list(queryWrapper);
    }
}
