package com.ruoyi.web.controller.callback;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.idgen.IdGenerate;
import com.ruoyi.market.constants.AdvertisingConstants;
import com.ruoyi.market.constants.DrawConstants;
import com.ruoyi.market.domain.MarketAdvertisingAward;
import com.ruoyi.market.domain.MarketAdvertisingLog;
import com.ruoyi.market.domain.vo.AdvertisingBackQueryVo;
import com.ruoyi.market.domain.vo.DrawCallBackQueryVo;
import com.ruoyi.market.service.MarketAwardLibraryService;
import com.ruoyi.market.service.MarketDrawActivityService;
import com.ruoyi.market.service.MarketAdvertisingAwardService;
import com.ruoyi.market.service.MarketAdvertisingLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Objects;

@Api(tags = "前端回调管理")
@RestController
@RequestMapping("/callback")
public class CallBackController{

    @Autowired
    RedisCache redisCache;

    @Autowired
    MarketDrawActivityService marketDrawActivityService;

    @Resource
    private MarketAdvertisingLogService marketAdvertisingLogService;

    @Resource
    private MarketAdvertisingAwardService marketAdvertisingAwardService;

    @Resource
    private MarketAwardLibraryService marketAwardLibraryService;


    @ApiOperation("抽奖回调")
    @PostMapping("drawCallback")
    public AjaxResult drawCallback(@RequestBody DrawCallBackQueryVo drawCallBackVo){
        Object cacheObject = redisCache.getCacheObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getUserId());
        //找不到redis数据 处理
        if(Objects.isNull(cacheObject)){
            redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getUserId());
            throw new RuntimeException("请求超时");
        }
        if(!drawCallBackVo.getResult().toString().equals(cacheObject.toString())){
            redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getUserId());
            throw new RuntimeException("数据异常");
        }

        //清空redis数据
        redisCache.deleteObject(DrawConstants.REDIS_DRAW_ACTIVITY_KEY + drawCallBackVo.getUserId());
        //回调奖品发放
        marketDrawActivityService.userDrawOver(drawCallBackVo);
        return AjaxResult.success("抽奖完成");
    }

    @ApiOperation("广告回调")
    @PostMapping("advertisingCallback")
    public AjaxResult advertisingCallback(@RequestBody AdvertisingBackQueryVo advertisingBackQueryVo) {
        // 回调生成用户观看广告记录
        MarketAdvertisingLog marketAdvertisingLog = new MarketAdvertisingLog();
        BeanUtils.copyProperties(advertisingBackQueryVo, marketAdvertisingLog);
        marketAdvertisingLog.setId(IdGenerate.nextId());
        marketAdvertisingLog.setCreateTime(LocalDateTime.now());
        marketAdvertisingLogService.save(marketAdvertisingLog);
        Object cacheObject = redisCache.getCacheObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
        //找不到redis数据 处理
        if(Objects.isNull(cacheObject)){
            redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
            throw new RuntimeException("请求超时");
        }
        if(!advertisingBackQueryVo.getVideoId().equals(cacheObject.toString())){
            redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
            throw new RuntimeException("数据异常");
        }
        redisCache.deleteObject(AdvertisingConstants.REDIS_ADVERTISING_KEY + advertisingBackQueryVo.getUserId());
        //暂时。。。。。
        if (advertisingBackQueryVo.getIsCatOver().equals("1")) {
            marketAwardLibraryService.adsGrantAward(advertisingBackQueryVo.getVideoId(), advertisingBackQueryVo.getUserId());
            return AjaxResult.success("奖励已发放");
        }
        return AjaxResult.success("广告已播放");
    }
}
